//$HeadURL: 
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2010 by:
 - Department of Geography, University of Bonn -
 and
 - lat/lon GmbH -

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
 ----------------------------------------------------------------------------*/
package org.deegree.portal.cataloguemanager.control;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.deegree.enterprise.control.ajax.AbstractListener;
import org.deegree.enterprise.control.ajax.WebEvent;
import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.framework.xml.XMLFragment;
import org.xml.sax.SAXException;

/**
 * 
 * 
 * 
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @author last edited by: $Author: jwilden $
 * 
 * @version $Revision: 25441 $, $Date: 2010-07-20 09:01:55 +0200 (Di, 20 Jul 2010) $
 */
abstract class AbstractMetadataListener extends AbstractListener {

    private static final ILogger LOG = LoggerFactory.getLogger( AbstractMetadataListener.class );

    private static final String CONFIG = "CONFIGURATION";

    public static final String SEARCHRESULTINFO = "SEARCHRESULTINFO";

    public static final String CURRENTSEARCH = "CURRENTSEARCH";
    
    public static final String COUNTER = "COUNTER";

    private static CatalogueManagerConfiguration config;

    /**
     * 
     * @param event
     * @return configuration of the metadata editor
     * @throws IOException
     */
    CatalogueManagerConfiguration getCatalogueManagerConfiguration( WebEvent event )
                            throws IOException {
        config = (CatalogueManagerConfiguration) event.getSession().getAttribute( CONFIG );
        if ( config == null ) {
            config = loadConfiguration( event );
            event.getSession().setAttribute( CONFIG, config );
        }
        return config;
    }

    private CatalogueManagerConfiguration loadConfiguration( WebEvent event )
                            throws IOException {
        if ( config == null ) {
            URL url = new File( event.getAbsolutePath( "WEB-INF/conf/cataloguemanager/cataloguemanager.xml" ) ).toURI().toURL();
            try {
                config = new CatalogueManagerConfiguration( url );
            } catch ( Exception e ) {
                e.printStackTrace();
                throw new IOException( e.getMessage() );
            }
        }
        return config;
    }

    /**
     * 
     * @param conf
     * @param hlevel
     * @return metadata template for passed hierarchy level
     * @throws IOException
     * @throws SAXException
     */
    protected XMLFragment loadMetadataTemplate( CatalogueManagerConfiguration conf, String hlevel )
                            throws IOException, SAXException {

        String tmp = conf.getTemplateURL( hlevel );
        URL url = conf.resolve( tmp );
        LOG.logDebug( "reading template for hierarchylevel: " + hlevel + ": ", url );
        return new XMLFragment( url );
    }

}
