//$HeadURL: 
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2010 by:
 - Department of Geography, University of Bonn -
 and
 - lat/lon GmbH -

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
 ----------------------------------------------------------------------------*/
package org.deegree.portal.cataloguemanager.control;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.deegree.enterprise.control.ajax.ResponseHandler;
import org.deegree.enterprise.control.ajax.WebEvent;
import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.framework.util.BootLogger;
import org.deegree.framework.util.HttpUtils;
import org.deegree.framework.util.TimeTools;
import org.deegree.framework.xml.XMLFragment;
import org.deegree.framework.xml.XMLParsingException;
import org.deegree.framework.xml.XMLTools;
import org.deegree.model.crs.CRSFactory;
import org.deegree.model.crs.CRSTransformationException;
import org.deegree.model.crs.CoordinateSystem;
import org.deegree.model.spatialschema.Envelope;
import org.deegree.model.spatialschema.GeometryFactory;
import org.deegree.ogcbase.CommonNamespaces;
import org.deegree.portal.cataloguemanager.model.Address;
import org.deegree.portal.cataloguemanager.model.CICitation;
import org.deegree.portal.cataloguemanager.model.CIDate;
import org.deegree.portal.cataloguemanager.model.CIDateTypeCode;
import org.deegree.portal.cataloguemanager.model.CIOnlineResource;
import org.deegree.portal.cataloguemanager.model.CIResponsibleParty;
import org.deegree.portal.cataloguemanager.model.Contact;
import org.deegree.portal.cataloguemanager.model.ContactInfo;
import org.deegree.portal.cataloguemanager.model.Date;
import org.deegree.portal.cataloguemanager.model.DateType;
import org.deegree.portal.cataloguemanager.model.DistributionFormat;
import org.deegree.portal.cataloguemanager.model.EXExtent;
import org.deegree.portal.cataloguemanager.model.EXGeographicBoundingBox;
import org.deegree.portal.cataloguemanager.model.EXTemporalExtent;
import org.deegree.portal.cataloguemanager.model.ExceptionBean;
import org.deegree.portal.cataloguemanager.model.Keyword;
import org.deegree.portal.cataloguemanager.model.Linkage;
import org.deegree.portal.cataloguemanager.model.MDDataIdentification;
import org.deegree.portal.cataloguemanager.model.MDDigitalTransferOptions;
import org.deegree.portal.cataloguemanager.model.MDDistribution;
import org.deegree.portal.cataloguemanager.model.MDFormat;
import org.deegree.portal.cataloguemanager.model.MDLegalConstraints;
import org.deegree.portal.cataloguemanager.model.MDMetadata;
import org.deegree.portal.cataloguemanager.model.MDReferenceSystem;
import org.deegree.portal.cataloguemanager.model.MDRepresentativeFraction;
import org.deegree.portal.cataloguemanager.model.MetadataBean;
import org.deegree.portal.cataloguemanager.model.Name;
import org.deegree.portal.cataloguemanager.model.OnLine;
import org.deegree.portal.cataloguemanager.model.ParentIdentifier;
import org.deegree.portal.cataloguemanager.model.Phone;
import org.deegree.portal.cataloguemanager.model.PointOfContact;
import org.deegree.portal.cataloguemanager.model.ResourceConstraints;
import org.deegree.portal.cataloguemanager.model.SpatialResolution;
import org.deegree.portal.cataloguemanager.model.TimePeriod;
import org.deegree.portal.cataloguemanager.model.TransferOptions;
import org.deegree.portal.cataloguemanager.model.Version;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * 
 * 
 * 
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @author last edited by: $Author: apoth $
 * 
 * @version $Revision: 30690 $, $Date: 2011-05-06 10:23:46 +0200 (Fr, 06 Mai 2011) $
 */
public class InsertMetadataListener extends AbstractMetadataListener {

    protected static CoordinateSystem crs;

    static {
        if ( crs == null ) {
            try {
                crs = CRSFactory.create( "EPSG:4326" );
            } catch ( Exception e ) {
                BootLogger.logError( e.getMessage(), e );
            }
        }
    }

    private static ILogger LOG = LoggerFactory.getLogger( InsertMetadataListener.class );

    protected CatalogueManagerConfiguration conf;

    @Override
    public void actionPerformed( WebEvent event, ResponseHandler responseHandler )
                            throws IOException {

        try {
            conf = getCatalogueManagerConfiguration( event );
            MetadataBean bean = (MetadataBean) event.getAsBean();
            JAXBContext jc = JAXBContext.newInstance( "org.deegree.portal.cataloguemanager.model" );
            Unmarshaller u = jc.createUnmarshaller();

            String tmp = conf.getTemplateURL( bean.getHlevel() );
            URL url = conf.resolve( tmp );
            MDMetadata mdMetadata = (MDMetadata) u.unmarshal( url );

            // metadata settings
            mdMetadata.getFileIdentifier().setCharacterString( bean.getIdentifier() );
            mdMetadata.getHierarchyLevel().getMDScopeCode().setCodeListValue( bean.getHlevel() );
            tmp = bean.getHlevel();
            if ( "series".equalsIgnoreCase( tmp ) ) {
                tmp = "datasetcollection";
            }
            if ( "dataset".equalsIgnoreCase( tmp ) && bean.getParentId() != null ) {
                ParentIdentifier pi = new ParentIdentifier();
                pi.setCharacterString( bean.getParentId() );
                mdMetadata.setParentIdentifier( pi );
            }
            mdMetadata.getHierarchyLevelName().setCharacterString( tmp );
            setContact( mdMetadata, bean );
            mdMetadata.getDateStamp().setDateTime( TimeTools.getISOFormattedTime() );
            if ( bean.getCrs() != null ) {
                MDReferenceSystem mdRef = mdMetadata.getReferenceSystemInfo().getMDReferenceSystem();
                mdRef.getReferenceSystemIdentifier().getRSIdentifier().getCode().setCharacterString( bean.getCrs() );
            }

            // data identification settings
            MDDataIdentification mdDataId = mdMetadata.getIdentificationInfo().getMDDataIdentification();
            mdDataId.getAbstract().setCharacterString( bean.getAbstract_() );
            mdDataId.setUuid( "_" + bean.getIdentifier() );
            mdDataId.setId( bean.getIdentifier() );
            setKeywords( mdDataId, bean );
            setPointOfContact( mdDataId, bean );
            mdDataId.getTopicCategory().setMDTopicCategoryCode( bean.getTopCat() );
            setExtent( mdDataId, bean );

            SpatialResolution spr = mdDataId.getSpatialResolution();
            if ( bean.getScale() == null || bean.getScale().trim().length() == 0 ) {
                mdDataId.setSpatialResolution( null );
            } else {
                MDRepresentativeFraction frac = spr.getMDResolution().getEquivalentScale().getMDRepresentativeFraction();
                frac.getDenominator().setInteger( Integer.parseInt( bean.getScale().trim() ) );
            }

            CICitation citation = mdDataId.getCitation().getCICitation();
            citation.getTitle().setCharacterString( bean.getDatasetTitle() );
            citation.getIdentifier().getMDIdentifier().getCode().setCharacterString( bean.getIdentifier() );

            // citation dates
            List<Date> dates = citation.getDate();
            dates.clear();
            if ( bean.getCreation() != null ) {
                addDate( dates, "creation", bean.getCreation() );
            }
            if ( bean.getPublication() != null ) {
                addDate( dates, "publication", bean.getPublication() );
            }
            if ( bean.getRevision() != null ) {
                addDate( dates, "revision", bean.getRevision() );
            }

            if ( bean.getTransferOnline() != null && bean.getTransferOnline().trim().length() > 0 ) {
                setDistributionInfo( mdMetadata, bean );
            } else {
                mdMetadata.setDistributionInfo( null );
            }

            // lineage
            if ( bean.getLineage() != null && bean.getLineage().trim().length() > 0 ) {
                mdMetadata.getDataQualityInfo().getDQDataQuality().getLineage().getLILineage().getStatement().setCharacterString(
                                                                                                                                  bean.getLineage() );
            } else {
                mdMetadata.setDataQualityInfo( null );
            }

            // resource constraints
            List<ResourceConstraints> list = mdMetadata.getIdentificationInfo().getMDDataIdentification().getResourceConstraints();
            if ( bean.getAccessConstraints() == null ) {
                list.clear();
            } else {
                MDLegalConstraints legal = null;
                for ( ResourceConstraints resourceConstraints : list ) {
                    if ( resourceConstraints.getMDLegalConstraints() != null ) {
                        legal = resourceConstraints.getMDLegalConstraints();
                        legal.getAccessConstraints().getMDRestrictionCode().setCodeListValue(
                                                                                              bean.getAccessConstraints() );
                        legal.getAccessConstraints().getMDRestrictionCode().setValue( bean.getAccessConstraints() );
                    }
                }
            }

            insertIntoCSW_( responseHandler, mdMetadata );

        } catch ( Exception e ) {
            LOG.logError( e.getMessage(), e );
            ExceptionBean eb = new ExceptionBean( getClass().getName(), e.getMessage() );
            responseHandler.writeAndClose( true, eb );
            return;
        }
    }

    private void setDistributionInfo( MDMetadata mdMetadata, MetadataBean bean ) {
        MDDistribution mddist = mdMetadata.getDistributionInfo().getMDDistribution();
        if ( mddist == null ) {
            mddist = new MDDistribution();
            TransferOptions to = new TransferOptions();
            MDDigitalTransferOptions dto = new MDDigitalTransferOptions();
            OnLine ol = new OnLine();
            CIOnlineResource ciolr = new CIOnlineResource();
            ciolr.setLinkage( new Linkage() );
            ol.setCIOnlineResource( ciolr );
            dto.setOnLine( ol );
            to.setMDDigitalTransferOptions( dto );
            mddist.setTransferOptions( to );
        }
        MDDigitalTransferOptions to = mddist.getTransferOptions().getMDDigitalTransferOptions();
        to.getOnLine().getCIOnlineResource().getLinkage().setURL( bean.getTransferOnline() );
        if ( bean.getTransferFormatName() != null && bean.getTransferFormatName().length() > 0 ) {
            DistributionFormat format = mdMetadata.getDistributionInfo().getMDDistribution().getDistributionFormat();
            if ( format == null ) {
                format = new DistributionFormat();
                MDFormat mdf = new MDFormat();
                mdf.setName( new Name() );
                mdf.setVersion( new Version() );
                format.setMDFormat( mdf );
            }
            format.getMDFormat().getName().setCharacterString( bean.getTransferFormatName() );
            if ( bean.getTransferFormatVersion() != null && bean.getTransferFormatVersion().length() > 0 ) {
                format.getMDFormat().getVersion().setCharacterString( bean.getTransferFormatVersion() );
            } else {
                format.getMDFormat().getVersion().setCharacterString( "unknown" );
            }
        } else {
            mdMetadata.getDistributionInfo().getMDDistribution().setDistributionFormat( null );
        }
    }

    @SuppressWarnings("unchecked")
    private String insertIntoCSW_( ResponseHandler responseHandler, MDMetadata mdMetadata )
                            throws JAXBException, IOException, HttpException, SAXException, XMLParsingException {

        JAXBContext jc = JAXBContext.newInstance( "org.deegree.portal.cataloguemanager.model" );
        Marshaller m = jc.createMarshaller();
        ByteArrayOutputStream bos = new ByteArrayOutputStream( 10000 );
        m.marshal( mdMetadata, bos );
        XMLFragment md = new XMLFragment();
        md.load( new ByteArrayInputStream( bos.toByteArray() ), XMLFragment.DEFAULT_URL );

        URL url = getClass().getResource( "insert.xml" );
        XMLFragment insert = new XMLFragment( url );
        Element elem = XMLTools.getElement( insert.getRootElement(), "csw202:Insert",
                                            CommonNamespaces.getNamespaceContext() );
        XMLTools.copyNode( md.getRootElement().getOwnerDocument(), elem );

        Enumeration<String> en = ( (HttpServletRequest) getRequest() ).getHeaderNames();
        Map<String, String> map = new HashMap<String, String>();
        while ( en.hasMoreElements() ) {
            String name = (String) en.nextElement();
            if ( !name.equalsIgnoreCase( "accept-encoding" ) && !name.equalsIgnoreCase( "content-length" )
                 && !name.equalsIgnoreCase( "user-agent" ) ) {
                map.put( name, ( (HttpServletRequest) getRequest() ).getHeader( name ) );
            }
        }
        HttpMethod post = HttpUtils.performHttpPost( conf.getCatalogueURL(), insert, 60000, null, null, map );
        String s = post.getResponseBodyAsString();
        if ( s.toLowerCase().indexOf( "exception" ) > -1 ) {
            ExceptionBean eb = new ExceptionBean( getClass().getName(), "insert failed" );
            responseHandler.writeAndClose( true, eb );
        } else {
            responseHandler.writeAndClose( "insert performed" );
        }

        return s;
    }

    protected void setContact( MDMetadata mdMetadata, MetadataBean bean ) {
        Contact contact = mdMetadata.getContact();
        CIResponsibleParty respParty = contact.getCIResponsibleParty();
        respParty.getIndividualName().setCharacterString( bean.getContactIndividualName() );
        respParty.getOrganisationName().setCharacterString( bean.getContactOrganisationName() );
        respParty.getRole().getCIRoleCode().setCodeListValue( bean.getContactRole() );
        ContactInfo ci = respParty.getContactInfo();
        Phone phone = ci.getCIContact().getPhone();
        phone.getCITelephone().getFacsimile().setCharacterString( bean.getContactFacsimile() );
        phone.getCITelephone().getVoice().setCharacterString( bean.getContactVoice() );
        Address address = ci.getCIContact().getAddress();
        address.getCIAddress().getCity().setCharacterString( bean.getContactCity() );
        address.getCIAddress().getCountry().setCharacterString( bean.getContactCountry() );
        address.getCIAddress().getDeliveryPoint().setCharacterString( bean.getContactDeliveryPoint() );
        address.getCIAddress().getElectronicMailAddress().setCharacterString( bean.getContactEmailAddress() );
        address.getCIAddress().getPostalCode().setCharacterString( bean.getContactPostalCode() );
    }

    protected void setExtent( MDDataIdentification mdDataId, MetadataBean bean )
                            throws CRSTransformationException {
        EXExtent extent = mdDataId.getExtent().getEXExtent();
        extent.getDescription().setCharacterString( getGeogrDesc( bean.getGeogrDescription() ) );
        EXTemporalExtent te = extent.getTemporalElement().getEXTemporalExtent();
        TimePeriod tp = te.getExtent().getTimePeriod();

        tp.setBeginPosition( bean.getBegin() );
        tp.setEndPosition( bean.getEnd() );
        EXGeographicBoundingBox bbox = extent.getGeographicElement().getEXGeographicBoundingBox();
        Envelope env = getEnvelope( bean.getGeogrDescription() );
        bbox.getWestBoundLongitude().setDecimal( (float) env.getMin().getX() );
        bbox.getSouthBoundLatitude().setDecimal( (float) env.getMin().getY() );
        bbox.getEastBoundLongitude().setDecimal( (float) env.getMax().getX() );
        bbox.getNorthBoundLatitude().setDecimal( (float) env.getMax().getY() );
    }

    /**
     * @param geographicIdentifier
     * @return
     */
    private String getGeogrDesc( String geographicIdentifier ) {
        List<SpatialExtent> extents = conf.getSpatialExtents();
        for ( SpatialExtent spatialExtent : extents ) {
            if ( spatialExtent.getId().equalsIgnoreCase( geographicIdentifier ) ) {
                return spatialExtent.getName();
            }
        }
        return "-";
    }

    protected Envelope getEnvelope( String geogrDescription )
                            throws CRSTransformationException {
        List<SpatialExtent> spatialExtents = conf.getSpatialExtents();
        String bbox = "-180,-90,180,90";
        for ( SpatialExtent spatialExtent : spatialExtents ) {
            if ( spatialExtent.getId().equals( geogrDescription ) ) {
                bbox = spatialExtent.getBbox();
                break;
            }
        }
        return GeometryFactory.createEnvelope( bbox, crs );
    }

    protected void setPointOfContact( MDDataIdentification mdDataId, MetadataBean bean ) {
        PointOfContact poc = mdDataId.getPointOfContact();
        CIResponsibleParty respParty = poc.getCIResponsibleParty();
        respParty.getIndividualName().setCharacterString( bean.getPocIndividualName() );
        respParty.getOrganisationName().setCharacterString( bean.getPocOrganisationName() );
        respParty.getRole().getCIRoleCode().setCodeListValue( bean.getPocRole() );
        ContactInfo ci = respParty.getContactInfo();
        Phone phone = ci.getCIContact().getPhone();
        phone.getCITelephone().getFacsimile().setCharacterString( bean.getPocFacsimile() );
        phone.getCITelephone().getVoice().setCharacterString( bean.getPocVoice() );
        Address address = ci.getCIContact().getAddress();
        address.getCIAddress().getCity().setCharacterString( bean.getPocCity() );
        address.getCIAddress().getCountry().setCharacterString( bean.getPocCountry() );
        address.getCIAddress().getDeliveryPoint().setCharacterString( bean.getPocDeliveryPoint() );
        address.getCIAddress().getElectronicMailAddress().setCharacterString( bean.getPocEmailAddress() );
        address.getCIAddress().getPostalCode().setCharacterString( bean.getPocPostalCode() );
    }

    @SuppressWarnings("unchecked")
    protected void setKeywords( MDDataIdentification mdDataId, MetadataBean bean ) {
        List<Keyword> list = mdDataId.getDescriptiveKeywords().getMDKeywords().getKeyword();
        list.clear();
        List<String> kw = bean.getKeywords();
        for ( String keyword : kw ) {
            Keyword key = new Keyword();
            key.setCharacterString( keyword );
            list.add( key );
        }
        Keyword key = new Keyword();
        key.setCharacterString( bean.getInspireDataTheme() );
        list.add( key );
    }

    protected void addDate( List<Date> dates, String type, String value ) {
        Date date = new Date();
        CIDate cidate = new CIDate();
        cidate.setDate( date );
        DateType dt = new DateType();
        CIDateTypeCode dtc = new CIDateTypeCode();
        dtc.setCodeList( "http://www.isotc211.org/2005/resources/codeList.xml#CI_DateTypeCode" );
        dtc.setCodeListValue( type );
        dt.setCIDateTypeCode( dtc );
        cidate.setDateType( dt );
        Date dateValue = new Date();
        dateValue.setDateTime( value + "T00:00:00Z" );
        cidate.setDate( dateValue );
        date.setCIDate( cidate );
        dates.add( date );
    }

}
